//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/ImportDataWidgets/SamplesTreeModel.cpp
//! @brief     Implements class SamplesTreeModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/SamplesTreeModel.h"
#include "GUI/Application/Application.h"
#include "GUI/Models/GUIExamplesFactory.h"
#include "GUI/Models/ModelUtils.h"
#include "GUI/Models/MultiLayerItem.h"
#include "GUI/Models/SampleModel.h"
#include "GUI/mainwindow/projectmanager.h"
#include <QApplication>
#include <QFontMetrics>
#include <QIcon>

SamplesTreeModel::SamplesTreeModel(QObject* parent, SampleModel* model)
    : QAbstractItemModel(parent), m_sampleModel(model)
{
    connect(m_sampleModel, &SampleModel::modelAboutToBeReset, this, &SamplesTreeModel::clear,
            Qt::UniqueConnection);
}

void SamplesTreeModel::clear()
{
    beginResetModel();
    endResetModel();
}

MultiLayerItem* SamplesTreeModel::topMostItem() const
{
    return m_sampleModel->topItem<MultiLayerItem>();
}

QModelIndex SamplesTreeModel::index(int row, int column, const QModelIndex& parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    if (!parent.isValid())
        return createIndex(row, column, nullptr);

    return createIndex(row, column, m_sampleModel->multiLayerItems()[row]);
}

QModelIndex SamplesTreeModel::parent(const QModelIndex& index) const
{
    if (!index.isValid())
        return QModelIndex();

    if (index.internalPointer() == nullptr) // index is headline => no parent
        return QModelIndex();

    return createIndex(0, 0, nullptr);
}

int SamplesTreeModel::columnCount(const QModelIndex& /*parent*/) const
{
    return 1;
}

int SamplesTreeModel::rowCount(const QModelIndex& parent) const
{
    if (!parent.isValid())
        return 1;

    // parent is a headline
    if (isHeadline(parent))
        return m_sampleModel->multiLayerItems().size();

    return 0; // parent is an item
}

QVariant SamplesTreeModel::data(const QModelIndex& index, int role) const
{
    if (isHeadline(index)) {
        const QString title = "Samples";

        switch (role) {
        case Qt::DisplayRole:
            return title;

        case Qt::FontRole: {
            QFont f(QApplication::font());
            f.setPointSize(f.pointSize() * 1.5);
            f.setBold(true);
            return f;
        }

        case Qt::SizeHintRole: {
            QFont f(QApplication::font());
            f.setPointSize(f.pointSize() * 1.5);
            f.setBold(true);
            QSize s = QFontMetrics(f).boundingRect(title).size();
            return QSize(s.width() * 2, s.height() * 2);
        }

        case Qt::TextAlignmentRole:
            return QVariant(Qt::AlignLeft | Qt::AlignVCenter);

        case Qt::BackgroundRole:
            return baApp->styleSheetPalette().headlineBackground;

        case Qt::ForegroundRole:
            return baApp->styleSheetPalette().headlineText;

        default:
            return QVariant();
        }
    }

    const auto item = itemForIndex(index);

    if (role == Qt::ToolTipRole)
        return item->description();

    if (role == Qt::DisplayRole)
        return item->itemName();

    if (role == Qt::EditRole)
        return item->itemName();

    if (role == Qt::DecorationRole)
        return QIcon(":/SampleDesignerToolbox/images/sample_layers2.png");

    return QVariant();
}

Qt::ItemFlags SamplesTreeModel::flags(const QModelIndex& index) const
{
    if (isHeadline(index) || !index.isValid())
        return Qt::NoItemFlags;

    auto f = QAbstractItemModel::flags(index);
    f |= Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled;

    return f;
}

bool SamplesTreeModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if (!index.isValid())
        return false;

    if (role == Qt::EditRole && index.column() == 0) {
        itemForIndex(index)->setItemName(value.toString());
        emit dataChanged(index, index);
        return true;
    }

    if (role == Qt::ToolTipRole && index.column() == 0) {
        itemForIndex(index)->setDescription(value.toString());
        emit dataChanged(index, index);
        return true;
    }

    return false;
}

MultiLayerItem* SamplesTreeModel::itemForIndex(const QModelIndex& index) const
{
    if (!index.isValid())
        return nullptr;

    return reinterpret_cast<MultiLayerItem*>(index.internalPointer());
}

QModelIndex SamplesTreeModel::indexForItem(MultiLayerItem* item) const
{
    if (item == nullptr)
        return QModelIndex();

    if (auto row = m_sampleModel->multiLayerItems().indexOf(item); row >= 0)
        return createIndex(row, 0, item);

    return QModelIndex();
}

void SamplesTreeModel::removeSample(MultiLayerItem* item)
{
    QModelIndex index = indexForItem(item);
    if (!index.isValid())
        return;

    beginRemoveRows(index.parent(), index.row(), index.row());
    m_sampleModel->removeMultiLayer(item);
    endRemoveRows();
}

bool SamplesTreeModel::isHeadline(const QModelIndex& index) const
{
    if (!index.isValid())
        return false;

    return index.internalPointer() == nullptr;
}

QModelIndex SamplesTreeModel::createSample()
{
    using namespace GUI::Model::ItemUtils;

    const int row = m_sampleModel->multiLayerItems().size();
    beginInsertRows(index(0, 0), row, row);
    auto multilayer_item = m_sampleModel->insertItem<MultiLayerItem>();
    multilayer_item->setItemName(suggestName(topItemNames(m_sampleModel), "Sample"));
    endInsertRows();
    return indexForItem(multilayer_item);
}

QModelIndex SamplesTreeModel::createSampleFromExamples(const QString& className,
                                                       const QString& title,
                                                       const QString& description)
{
    const int row = m_sampleModel->multiLayerItems().size();
    beginInsertRows(index(0, 0), row, row);

    auto* sample = dynamic_cast<MultiLayerItem*>(GUIExamplesFactory::createSampleItems(
        className, m_sampleModel, ProjectManager::instance()->document()->materialModel()));
    sample->setItemName(title);
    sample->setDescription(description);

    endInsertRows();
    return indexForItem(sample);
}
